@@ -49,6 +49,8 @@ static gpointer grab_object = NULL;
 
 struct GSGrabPrivate
 {
+        GDBusConnection *session_bus;
+
         guint      mouse_hide_cursor : 1;
         GdkWindow *mouse_grab_window;
         GdkWindow *keyboard_grab_window;
@@ -58,31 +60,6 @@ struct GSGrabPrivate
         GtkWidget *invisible;
 };
 
-static GdkCursor *
-get_cursor (void)
-{
-        GdkBitmap *empty_bitmap;
-        GdkCursor *cursor;
-        GdkColor   useless;
-        char       invisible_cursor_bits [] = { 0x0 };
-
-        useless.red = useless.green = useless.blue = 0;
-        useless.pixel = 0;
-
-        empty_bitmap = gdk_bitmap_create_from_data (NULL,
-                                                    invisible_cursor_bits,
-                                                    1, 1);
-
-        cursor = gdk_cursor_new_from_pixmap (empty_bitmap,
-                                             empty_bitmap,
-                                             &useless,
-                                             &useless, 0, 0);
-
-        g_object_unref (empty_bitmap);
-
-        return cursor;
-}
-
 static const char *
 grab_string (int status)
 {
@@ -120,7 +97,7 @@ xorg_lock_smasher_set_active (GSGrab  *grab,
 {
         int status, event, error;
 
-	if (!XF86MiscQueryExtension (GDK_DISPLAY (), &event, &error)) {
+	if (!XF86MiscQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &event, &error)) {
 		gs_debug ("No XFree86-Misc extension present");
 		return;
 	}
@@ -133,22 +110,27 @@ xorg_lock_smasher_set_active (GSGrab  *grab,
 
         gdk_error_trap_push ();
 
-        status = XF86MiscSetGrabKeysState (GDK_DISPLAY (), active);
+        status = XF86MiscSetGrabKeysState (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), active);
 
         gdk_display_sync (gdk_display_get_default ());
-        gdk_error_trap_pop ();
+        error = gdk_error_trap_pop ();
 
         if (active && status == MiscExtGrabStateAlready) {
                 /* shut up, consider this success */
                 status = MiscExtGrabStateSuccess;
         }
 
-        gs_debug ("XF86MiscSetGrabKeysState(%s) returned %s\n",
-                  active ? "on" : "off",
-                  (status == MiscExtGrabStateSuccess ? "MiscExtGrabStateSuccess" :
-                   status == MiscExtGrabStateLocked  ? "MiscExtGrabStateLocked"  :
-                   status == MiscExtGrabStateAlready ? "MiscExtGrabStateAlready" :
-                   "unknown value"));
+        if (error == Success) {
+                gs_debug ("XF86MiscSetGrabKeysState(%s) returned %s\n",
+                          active ? "on" : "off",
+                          (status == MiscExtGrabStateSuccess ? "MiscExtGrabStateSuccess" :
+                           status == MiscExtGrabStateLocked  ? "MiscExtGrabStateLocked"  :
+                           status == MiscExtGrabStateAlready ? "MiscExtGrabStateAlready" :
+                           "unknown value"));
+        } else {
+                gs_debug ("XF86MiscSetGrabKeysState(%s) failed with error code %d\n",
+                          active ? "on" : "off", error);
+        }
 }
 #else
 static void
@@ -201,7 +183,7 @@ gs_grab_get_mouse (GSGrab    *grab,
         g_return_val_if_fail (window != NULL, FALSE);
         g_return_val_if_fail (screen != NULL, FALSE);
 
-        cursor = get_cursor ();
+        cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
 
         gs_debug ("Grabbing mouse widget=%X", (guint32) GDK_WINDOW_XID (window));
         status = gdk_pointer_grab (window, TRUE, 0, NULL,
@@ -404,12 +386,11 @@ gs_grab_nuke_focus (void)
 
         gdk_error_trap_push ();
 
-        XGetInputFocus (GDK_DISPLAY (), &focus, &rev);
+        XGetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &focus, &rev);
 
-        XSetInputFocus (GDK_DISPLAY (), None, RevertToNone, CurrentTime);
+        XSetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), None, RevertToNone, CurrentTime);
 
-        gdk_display_sync (gdk_display_get_default ());
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 void
@@ -427,6 +408,39 @@ gs_grab_release (GSGrab *grab)
         gdk_flush ();
 }
 
+/* The GNOME 3 Shell holds an X grab when we're in the overview;
+ * ask it to bounce out before we try locking the screen.
+ */
+static void
+request_shell_exit_overview (GSGrab *grab)
+{
+        GDBusMessage *reply;
+        GDBusMessage *message;
+        GError *error = NULL;
+
+        /* Shouldn't happen, but... */
+        if (!grab->priv->session_bus)
+                return;
+
+        message = g_dbus_message_new_method_call ("org.gnome.Shell",
+                                                  "/org/gnome/Shell",
+                                                  "org.freedesktop.DBus.Properties",
+                                                  "Set");
+        g_dbus_message_set_body (message,
+                                 g_variant_new ("(ssv)",
+                                                "org.gnome.Shell",
+                                                "OverviewActive",
+                                                g_variant_new ("b",
+                                                               FALSE)));
+
+        g_dbus_connection_send_message (grab->priv->session_bus,
+                                        message,
+                                        G_DBUS_SEND_MESSAGE_FLAGS_NONE,
+                                        NULL,
+                                        NULL);
+        g_object_unref (message);
+}
+
 gboolean
 gs_grab_grab_window (GSGrab    *grab,
                      GdkWindow *window,
@@ -439,6 +453,9 @@ gs_grab_grab_window (GSGrab    *grab,
         int      retries = 4;
         gboolean focus_fuckus = FALSE;
 
+        /* First, have stuff we control in GNOME un-grab */
+        request_shell_exit_overview (grab);
+
  AGAIN:
 
         for (i = 0; i < retries; i++) {
@@ -540,7 +557,7 @@ gs_grab_grab_offscreen (GSGrab *grab,
         gs_debug ("Grabbing an offscreen window");
 
         screen = gtk_invisible_get_screen (GTK_INVISIBLE (grab->priv->invisible));
-        res = gs_grab_grab_window (grab, grab->priv->invisible->window, screen, hide_cursor);
+        res = gs_grab_grab_window (grab, gtk_widget_get_window (grab->priv->invisible), screen, hide_cursor);
 
         return res;
 }
@@ -584,6 +601,8 @@ gs_grab_init (GSGrab *grab)
 {
         grab->priv = GS_GRAB_GET_PRIVATE (grab);
 
+        grab->priv->session_bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+
         grab->priv->mouse_hide_cursor = FALSE;
         grab->priv->invisible = gtk_invisible_new ();
         gtk_widget_show (grab->priv->invisible);
@@ -599,6 +618,8 @@ gs_grab_finalize (GObject *object)
 
         grab = GS_GRAB (object);
 
+        g_object_unref (grab->priv->session_bus);
+
         g_return_if_fail (grab->priv != NULL);
 
         gtk_widget_destroy (grab->priv->invisible);
